@@ -9,6 +9,8 @@ struct my_functions
 
    void member_function() { std::printf("Member Function!\n"); }
 
+   void const_member_function() const { std::printf("const Member Function!\n"); }
+
    std::string dump() { return "dumping\n"; }
 
    void input(int i) { std::printf("i: %d\n", i); }
@@ -18,7 +20,8 @@ template <>
 struct glz::meta<my_functions>
 {
    using T = my_functions;
-   static constexpr auto value = object("hi", &T::hello, &T::member_function, &T::dump, &T::input);
+   static constexpr auto value =
+      object("hi", &T::hello, &T::member_function, &T::const_member_function, &T::dump, &T::input);
 };
 
 struct four_t
