@@ -1,3 +1,4 @@
+#include <algorithm>
 #include <cstdint>
 #include <glaze/json.hpp>
 #include <glaze/json/schema.hpp>
@@ -83,98 +84,98 @@ auto expect_property(const test_case& test, std::string_view key, Value value)
 
 suite schema_attributes = [] {
    "parsing"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect(test.obj.has_value()) << format_error(!test.obj.has_value() ? test.obj.error() : glz::error_ctx{},
                                                    test.schema_str);
       expect[test.obj.has_value()];
    };
    "description"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::description>(test, "variable", std::string{"this is a description"});
    };
    "deprecated"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::deprecated>(test, "variable", true);
    };
    "readOnly"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::readOnly>(test, "variable", true);
    };
    "writeOnly"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::writeOnly>(test, "variable", true);
    };
    "minLength"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::minLength>(test, "variable", std::uint64_t{1});
    };
    "maxLength"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::maxLength>(test, "variable", std::uint64_t{2});
    };
    "pattern"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::pattern>(test, "variable", std::string_view{"[a-z]+"});
    };
    "format"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::format>(test, "variable", glz::detail::defined_formats::hostname);
    };
    "minimum"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::minimum>(test, "variable", std::int64_t{1});
    };
    "maximum"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::maximum>(test, "variable", std::int64_t{2});
    };
    "exclusiveMinimum"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::exclusiveMinimum>(test, "variable", std::int64_t{1});
    };
    "exclusiveMaximum"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::exclusiveMaximum>(test, "variable", std::int64_t{2});
    };
    "multipleOf"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::multipleOf>(test, "variable", std::int64_t{3});
    };
    "minProperties"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::minProperties>(test, "variable", std::uint64_t{4});
    };
    "maxProperties"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::maxProperties>(test, "variable", (std::numeric_limits<std::uint64_t>::max)());
    };
    "minItems"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::minItems>(test, "variable", std::uint64_t{1});
    };
    "maxItems"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::maxItems>(test, "variable", std::uint64_t{2});
    };
    "minContains"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::minContains>(test, "variable", std::uint64_t{1});
    };
    "maxContains"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::maxContains>(test, "variable", std::uint64_t{2});
    };
    "uniqueItems"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::uniqueItems>(test, "variable", true);
    };
    "extUnits"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::ExtUnits>(test, "variable",
                                               glz::detail::ExtUnits{.unitAscii = "m^2", .unitUnicode = "m²"});
    };
    "extAdvanced"_test = [] {
-      test_case const test{};
+      const test_case test{};
       expect_property<&glz::schema::ExtAdvanced>(test, "variable", true);
    };
 };
@@ -265,6 +266,48 @@ auto read_json_ignore_unknown(auto&& value, auto&& buffer)
    return read<glz::opts{.error_on_unknown_keys = false}>(value, std::forward<decltype(buffer)>(buffer), ctx);
 }
 
+struct required_meta
+{
+   int a{};
+   int reserved_1{};
+   int reserved_2{};
+   int b{};
+};
+
+template <>
+struct glz::meta<required_meta>
+{
+   static constexpr bool requires_key(const std::string_view key, const bool is_nullable)
+   {
+      if (key.starts_with("reserved")) {
+         return false;
+      }
+      return !is_nullable;
+   }
+};
+
+struct error_on_missing_keys_test
+{
+   std::optional<int> unimportant{};
+   int important{};
+};
+
+// Test types for demonstrating error_on_missing_keys with nested objects
+struct Address
+{
+   std::string street{};
+   std::string city{};
+   std::optional<std::string> apartment{}; // nullable - will NOT be required
+};
+
+struct Person
+{
+   std::string name{};
+   int age{};
+   std::optional<std::string> nickname{}; // nullable - will NOT be required
+   Address address{};
+};
+
 #if 0
 struct invalid_name
 {
@@ -338,7 +381,7 @@ suite schema_tests = [] {
       std::string schema_str = glz::write_json_schema<std::monostate>().value_or("error");
       // reading null will of course leave the std::optional as empty, therefore check if
       // null is actually written in the schema
-      expect(schema_str == R"({"type":["null"],"$defs":{},"const":null})");
+      expect(schema_str == R"({"type":["null"],"$defs":{},"title":"std::monostate","const":null})");
    };
 
    "enum oneOf has title and constant"_test = [] {
@@ -377,6 +420,58 @@ suite schema_tests = [] {
       expect[(obj.attributes.maxItems.has_value())];
       expect(obj.attributes.maxItems.value() == 42);
    };
+
+   "required_key meta is correctly used"_test = [] {
+      std::string schema_str = glz::write_json_schema<required_meta>().value_or("error");
+      expect(
+         schema_str ==
+         R"({"type":["object"],"properties":{"a":{"$ref":"#/$defs/int32_t"},"b":{"$ref":"#/$defs/int32_t"},"reserved_1":{"$ref":"#/$defs/int32_t"},"reserved_2":{"$ref":"#/$defs/int32_t"}},"additionalProperties":false,"$defs":{"int32_t":{"type":["integer"],"minimum":-2147483648,"maximum":2147483647}},"required":["a","b"],"title":"required_meta"})");
+   };
+
+   "Opts.error_on_missing_keys as fallback"_test = [] {
+      using T = error_on_missing_keys_test;
+      std::string schema_str_req =
+         glz::write_json_schema<T, glz::opts{.error_on_missing_keys = true}>().value_or("error");
+      std::string schema_str_nreq =
+         glz::write_json_schema<T, glz::opts{.error_on_missing_keys = false}>().value_or("error");
+
+      expect(
+         schema_str_req ==
+         R"({"type":["object"],"properties":{"important":{"$ref":"#/$defs/int32_t"},"unimportant":{"$ref":"#/$defs/std::optional<int32_t>"}},"additionalProperties":false,"$defs":{"int32_t":{"type":["integer"],"minimum":-2147483648,"maximum":2147483647},"std::optional<int32_t>":{"type":["integer","null"],"minimum":-2147483648,"maximum":2147483647}},"required":["important"],"title":"error_on_missing_keys_test"})");
+
+      expect(
+         schema_str_nreq ==
+         R"({"type":["object"],"properties":{"important":{"$ref":"#/$defs/int32_t"},"unimportant":{"$ref":"#/$defs/std::optional<int32_t>"}},"additionalProperties":false,"$defs":{"int32_t":{"type":["integer"],"minimum":-2147483648,"maximum":2147483647},"std::optional<int32_t>":{"type":["integer","null"],"minimum":-2147483648,"maximum":2147483647}},"title":"error_on_missing_keys_test"})");
+   };
+
+   // Demonstrates using error_on_missing_keys to mark all non-nullable fields as required in JSON schema
+   "error_on_missing_keys marks all non-nullable fields as required"_test = [] {
+      // Generate schema with all non-nullable fields marked as required
+      auto schema = glz::write_json_schema<Person, glz::opts{.error_on_missing_keys = true}>().value_or("error");
+
+      // Parse the schema to verify the required fields
+      glz::detail::schematic obj{};
+      auto err = glz::read<glz::opts{.error_on_unknown_keys = false}>(obj, schema);
+      expect(!err) << glz::format_error(err, schema);
+
+      // Verify Person has required fields: name, age, address (but NOT nickname)
+      expect(obj.required.has_value());
+      auto& req = *obj.required;
+      expect(std::find(req.begin(), req.end(), "name") != req.end());
+      expect(std::find(req.begin(), req.end(), "age") != req.end());
+      expect(std::find(req.begin(), req.end(), "address") != req.end());
+      expect(std::find(req.begin(), req.end(), "nickname") == req.end()); // nullable, not required
+
+      // Verify Address definition also has required fields: street, city (but NOT apartment)
+      expect(obj.defs.has_value());
+      auto address_it = obj.defs->find("Address");
+      expect(address_it != obj.defs->end());
+      expect(address_it->second.required.has_value());
+      auto& addr_req = *address_it->second.required;
+      expect(std::find(addr_req.begin(), addr_req.end(), "street") != addr_req.end());
+      expect(std::find(addr_req.begin(), addr_req.end(), "city") != addr_req.end());
+      expect(std::find(addr_req.begin(), addr_req.end(), "apartment") == addr_req.end()); // nullable, not required
+   };
 };
 
 int main() { return 0; }
